@@ -4,7 +4,9 @@ use Moose::Role;
 
 use integer;
 use Math::Int64 qw/int64_to_number int64_to_net int64 net_to_int64/;
-use Math::BigInt;
+use Math::BigInt lib   => 'GMP';
+use Math::BigFloat lib => 'GMP';
+use Hessian::Exception;
 use POSIX qw/floor ceil/;
 use Switch;
 
@@ -32,27 +34,34 @@ sub read_integer {    #{{{
 sub read_long {    #{{{
     my ( $self, $hessian_data ) = @_;
     ( my $raw_octets = $hessian_data ) =~ s/^(?:L|\x77)(.*)/$1/;
-    my @chars = unpack 'C*', $raw_octets;
-    my $array_size = scalar @chars;
+    my @chars       = unpack 'C*', $raw_octets;
+    my $array_size  = scalar @chars;
     my $octet_count = scalar @chars;
     my $result =
         $octet_count == 1 ? _read_single_octet( $chars[0], 0xe0 )
       : $octet_count == 2 ? _read_double_octet( \@chars, 0xf8 )
       : $octet_count == 3 ? _read_triple_octet( \@chars, 0x3c )
-      : $octet_count == 4 ? _read_quadruple_octet( \@chars, )
-      :                     _read_full_long( $raw_octets); #\@chars );
+      : $octet_count == 4 ? Implementation::X->throw(
+        error => "32 bit longs
+      not currently supported."
+      )
+      : _read_full_long($raw_octets);    #\@chars );
+                                         #_read_quadruple_octet( \@chars,
     return $result;
 }    #}}}
 
 sub read_double {    #{{{
     my ( $self, $octet ) = @_;
     my $double_value =
-           $octet =~ /\x{5b}/                    ? 0.0
-         : $octet =~ /\x{5c}/                    ? 1.0
-         : $octet =~ /(?: \x{5d} | \x{5e} ) .*/x ? _read_compact_double($octet)
-         : $octet =~ /\x5f/           ? _read_quadruple_octet_double($octet)
-      :                                     _read_full_double($octet);
-      return $double_value;
+        $octet =~ /\x{5b}/                    ? 0.0
+      : $octet =~ /\x{5c}/                    ? 1.0
+      : $octet =~ /(?: \x{5d} | \x{5e} ) .*/x ? _read_compact_double($octet)
+      : $octet =~ /\x5f/                      ? Implementation::X->throw(
+        error => "32 bit doubles not currently supported." )
+      : _read_full_double($octet);
+
+    #_read_quadruple_octet_double($octet)
+    return $double_value;
 }    #}}}
 
 sub _read_single_octet {    #{{{
@@ -76,25 +85,27 @@ sub _read_triple_octet {    #{{{
     return $integer;
 }    #}}}
 
-#sub _read_quadruple_long_octet {    #{{{
-#    my ( $bytes, $octet_shift ) = @_;
-#    my $big_int = Math::BigInt->new();
-#    my $shift_val = 0;
-#   my $index = 0; 
-#    foreach my $byte ( reverse @{$bytes} ) {
-#        $index++;
-#        my $shift_byte = Math::BigInt->new($byte);
-#        $shift_byte->bsub($octet_shift) if $index == 4;
-#        $shift_byte->blsft($shift_val);
-#        $big_int->badd($shift_byte);
-#        $shift_val += 8;
-#    }
-#    print "mantissa: ".$big_int->mantissa()."\n";
-#    return $big_int->bstr();
-#}    #}}}
+sub _read_quadruple_long_octet {    #{{{
+    my ( $bytes, $octet_shift ) = @_;
+    my $big_int   = Math::BigInt->new();
+    my $shift_val = 0;
+    my $index     = 0;
+    foreach my $byte ( reverse @{$bytes} ) {
+
+        #        $index++;
+        my $shift_byte = Math::BigInt->new($byte);
+
+        #        $shift_byte->bsub($octet_shift) if $index == 4;
+        $shift_byte->blsft($shift_val);
+        $big_int->badd($shift_byte);
+        $shift_val += 8;
+    }
+    return $big_int->bstr();
+}    #}}}
 
 sub _read_quadruple_octet {    #{{{
-    my $bytes     = shift;
+    my $bytes = shift;
+
     my $shift_val = 0;
     my $sum;
     foreach my $byte ( reverse @{$bytes} ) {
@@ -105,9 +116,9 @@ sub _read_quadruple_octet {    #{{{
 }    #}}}
 
 sub _read_full_long {    #{{{
-    my $string     = shift;
+    my $string    = shift;
     my $net_int64 = net_to_int64($string);
-    return "$net_int64"
+    return "$net_int64";
 }    #}}}
 
 sub _read_compact_double {    #{{{
@@ -119,22 +130,23 @@ sub _read_compact_double {    #{{{
     return $float;
 }    #}}}
 
-sub  _read_quadruple_octet_double { #{{{
+sub _read_quadruple_octet_double {    #{{{
     my $octets = shift;
     $octets =~ s/\x5f//;
-    my @chars = unpack 'C*', $octets;
-    my @hex_octets = 
-    map { sprintf "%#02x", $_} @chars;
-    my $hex_string = join "" => @hex_octets;
+
+    #     my @chars = unpack 'C*', $octets;
+    #     my @hex_octets =
+    #       map { sprintf "%#02x", $_ } @chars;
+    #     my $hex_string = join "" => @hex_octets;
 
     my $double;
-     $double = str2float($hex_string);
+
+    #     $double = str2float($hex_string);
     return $double;
-} #}}}
+}    #}}}
 
 sub _read_full_double {    #{{{
     my $double = shift;
-    print "processing a full double\n";
     ( my $octets = $double ) =~ s/(?:D ) (.*) /$1/x;
     my @chars = unpack 'C*', $octets;
     my $double_value = unpack 'F', pack 'C*', reverse @chars;
@@ -193,11 +205,11 @@ sub read_long_handle_chunk {    #{{{
             $data   = $self->read_from_inputhandle(2);
             $number = $self->read_long( $first_bit . $data );
         }
-        case /\x59/ {  
-           $data = $self->read_from_inputhandle(4);
-           $number = $self->read_long($data);
-            
-            }
+        case /\x59/ {
+            $data   = $self->read_from_inputhandle(4);
+            $number = $self->read_long($data);
+
+        }
         case /\x4c/ {
 
             #            read $input_handle, $data, 8;
@@ -228,24 +240,24 @@ sub read_double_handle_chunk {    #{{{
         }
         case /\x5e/ {
 
-            $data = $first_bit. $self->read_from_inputhandle(2);
+            $data = $first_bit . $self->read_from_inputhandle(2);
 
             #            read $input_handle, $data, 2;
         }
         case /\x5f/ {
 
             #            read $input_handle, $data, 4;
-            $data = $first_bit. $self->read_from_inputhandle(4);
+            $data = $first_bit . $self->read_from_inputhandle(4);
         }
         case /\x44/ {
             $first_bit = "";
 
             #            read $input_handle, $data, 8;
-            $data = $first_bit. $self->read_from_inputhandle(8);
+            $data = $first_bit . $self->read_from_inputhandle(8);
         }
 
     }
-    $number = $self->read_double(  $data );
+    $number = $self->read_double($data);
     return $number;
 }    #}}}
 
